=== modified file 'NEWS'
--- NEWS	2008-05-29 21:00:00 +0000
+++ NEWS	2008-05-30 00:35:19 +0000
@@ -34,6 +34,11 @@
 
   IMPROVEMENTS:
 
+    * ``bzr branch/push/pull -r XXX`` now have a helper function for finding
+      the revno of the new revision (``Graph.find_distance_to_null``). This
+      should make something like ``bzr branch -r -100`` in a shared, no-trees
+      repository much snappier. (John Arbash Meinel)
+
     * ``bzr ls`` now accepts ``-V`` as an alias for ``--versioned``. 
       (Jerad Cramp, #165086)
 
@@ -41,10 +46,12 @@
       ``Graph.find_differences`` to determine missing revisions without having
       to search the whole ancestry. (John Arbash Meinel, #174625)
 
-    * ``bzr branch/push/pull -r XXX`` now have a helper function for finding
-      the revno of the new revision (``Graph.find_distance_to_null``). This
-      should make something like ``bzr branch -r -100`` in a shared, no-trees
-      repository much snappier. (John Arbash Meinel)
+    * ``bzr uncommit`` now uses partial history access, rather than always
+      extracting the full revision history for a branch. This makes it
+      resolve the appropriate revisions much faster (in testing it drops
+      uncommit from 1.5s => 0.4s). It also means ``bzr log --short`` is one
+      step closer to not using full revision history.
+      (John Arbash Meinel, #172649)
 
   BUGFIXES:
 

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2008-05-19 18:56:45 +0000
+++ bzrlib/log.py	2008-05-30 00:22:33 +0000
@@ -335,8 +335,8 @@
 
     :return: A (mainline_revs, rev_nos, start_rev_id, end_rev_id) tuple.
     """
-    which_revs = _enumerate_history(branch)
-    if not which_revs:
+    branch_revno, branch_last_revision = branch.last_revision_info()
+    if branch_revno == 0:
         return None, None, None, None
 
     # For mainline generation, map start_revision and end_revision to 
@@ -349,7 +349,7 @@
     if start_revision is None:
         start_revno = 1
     else:
-        if isinstance(start_revision,RevisionInfo):
+        if isinstance(start_revision, RevisionInfo):
             start_rev_id = start_revision.rev_id
             start_revno = start_revision.revno or 1
         else:
@@ -358,11 +358,11 @@
     
     end_rev_id = None
     if end_revision is None:
-        end_revno = len(which_revs)
+        end_revno = branch_revno
     else:
-        if isinstance(end_revision,RevisionInfo):
+        if isinstance(end_revision, RevisionInfo):
             end_rev_id = end_revision.rev_id
-            end_revno = end_revision.revno or len(which_revs)
+            end_revno = end_revision.revno or branch_revno
         else:
             branch.check_real_revno(end_revision)
             end_revno = end_revision
@@ -374,20 +374,29 @@
         raise BzrCommandError("Start revision must be older than "
                               "the end revision.")
 
-    # list indexes are 0-based; revisions are 1-based
-    cut_revs = which_revs[(start_revno-1):(end_revno)]
-    if not cut_revs:
+    if end_revno < start_revno:
         return None, None, None, None
+    cur_revno = branch_revno
+    rev_nos = {}
+    mainline_revs = []
+    for revision_id in branch.repository.iter_reverse_revision_history(
+                        branch_last_revision):
+        if cur_revno < start_revno:
+            # We have gone far enough, but we always add 1 more revision
+            rev_nos[revision_id] = cur_revno
+            mainline_revs.append(revision_id)
+            break
+        if cur_revno <= end_revno:
+            rev_nos[revision_id] = cur_revno
+            mainline_revs.append(revision_id)
+        cur_revno -= 1
+    else:
+        # We walked off the edge of all revisions, so we add a 'None' marker
+        mainline_revs.append(None)
 
-    # convert the revision history to a dictionary:
-    rev_nos = dict((k, v) for v, k in cut_revs)
+    mainline_revs.reverse()
 
     # override the mainline to look like the revision history.
-    mainline_revs = [revision_id for index, revision_id in cut_revs]
-    if cut_revs[0][0] == 1:
-        mainline_revs.insert(0, None)
-    else:
-        mainline_revs.insert(0, which_revs[start_revno-2][1])
     return mainline_revs, rev_nos, start_rev_id, end_rev_id
 
 

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2008-03-20 15:10:05 +0000
+++ bzrlib/uncommit.py	2008-05-29 22:15:28 +0000
@@ -35,6 +35,9 @@
     :param dry_run: Don't actually change anything
     :param verbose: Print each step as you do it
     :param revno: Remove back to this revision
+    :param local: If this branch is bound, only remove the revisions from the
+        local branch. If this branch is not bound, it is an error to pass
+        local=True.
     """
     unlockable = []
     try:
@@ -58,35 +61,42 @@
             if master is not None:
                 master.lock_write()
                 unlockable.append(master)
-        rh = branch.revision_history()
-        if master is not None and rh[-1] != master.last_revision():
+        old_revno, old_tip = branch.last_revision_info()
+        if master is not None and old_tip != master.last_revision():
             raise BoundBranchOutOfDate(branch, master)
         if revno is None:
-            revno = len(rh)
-        old_revno, old_tip = branch.last_revision_info()
-        new_revno = revno -1
+            revno = old_revno
+        new_revno = revno - 1
 
-        files_to_remove = []
-        for r in range(revno-1, len(rh)):
-            rev_id = rh.pop()
-            # NB: performance would be better using the revision graph rather
-            # than the whole revision.
-            rev = branch.repository.get_revision(rev_id)
+        revid_iterator = branch.repository.iter_reverse_revision_history(
+                            old_tip)
+        cur_revno = old_revno
+        new_revision_id = old_tip
+        graph = branch.repository.get_graph()
+        for rev_id in revid_iterator:
+            if cur_revno == new_revno:
+                new_revision_id = rev_id
+                break
+            if verbose:
+                print 'Removing revno %d: %s' % (cur_revno, rev_id)
+            cur_revno -= 1
+            parents = graph.get_parent_map([rev_id]).get(rev_id, None)
+            if not parents:
+                continue
             # When we finish popping off the pending merges, we want
             # them to stay in the order that they used to be.
             # but we pop from the end, so reverse the order, and
             # then get the order right at the end
-            pending_merges.extend(reversed(rev.parent_ids[1:]))
-            if verbose:
-                print 'Removing revno %d: %s' % (len(rh)+1, rev_id)
+            pending_merges.extend(reversed(parents[1:]))
+        else:
+            # We ran off the end of revisions, which means we should be trying
+            # to get to NULL_REVISION
+            new_revision_id = _mod_revision.NULL_REVISION
 
-        # Committing before we start removing files, because
-        # once we have removed at least one, all the rest are invalid.
         if not dry_run:
             if master is not None:
-                master.set_revision_history(rh)
-            branch.set_revision_history(rh)
-            new_tip = _mod_revision.ensure_null(branch.last_revision())
+                master.set_last_revision_info(new_revno, new_revision_id)
+            branch.set_last_revision_info(new_revno, new_revision_id)
             if master is None:
                 hook_local = None
                 hook_master = branch
@@ -94,14 +104,14 @@
                 hook_local = branch
                 hook_master = master
             for hook in Branch.hooks['post_uncommit']:
-                hook_new_tip = new_tip
+                hook_new_tip = new_revision_id
                 if hook_new_tip == _mod_revision.NULL_REVISION:
                     hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
                      hook_new_tip)
             if tree is not None:
-                if not _mod_revision.is_null(new_tip):
-                    parents = [new_tip]
+                if not _mod_revision.is_null(new_revision_id):
+                    parents = [new_revision_id]
                 else:
                     parents = []
                 parents.extend(reversed(pending_merges))

